import { Button, Input } from '@/lib/components';
import { useState } from 'react';
import { EmailProps } from './types';

export const EmailInput = ({
  placeholder,
  value = [],
  size = 'md',
  required = false,
  errorText,
  onEmailsChange,
  className,
  label,
}: EmailProps) => {
  const [emails, setEmails] = useState<string[]>(value);
  const [currentInput, setCurrentInput] = useState<string>('');

  const isValidEmail = (email: string): boolean => {
    const emailRegex = /^[^\s@]+@[^\s@]+\.[^\s@]+$/;
    return emailRegex.test(email);
  };

  const addEmail = () => {
    if (currentInput.trim() && isValidEmail(currentInput)) {
      const newEmails = [...emails, currentInput.trim()];
      setEmails(newEmails);
      setCurrentInput('');
      updateEmails(newEmails);
    }
  };

  const removeEmail = (index: number) => {
    const newEmails = emails.filter((_, i) => i !== index);
    setEmails(newEmails);
    updateEmails(newEmails);
  };

  const handleInputChange = (value: string) => {
    if (value.includes(' ')) {
      addEmail();
    } else {
      setCurrentInput(value);
    }
  };

  const updateEmails = (newEmails: string[]) => {
    setEmails(newEmails);
    if (onEmailsChange) {
      onEmailsChange(newEmails);
    }
  };

  return (
    <div className="w-full">
      <div className="flex flex-col gap-2 cursor-pointer m-0">
        {label && (
          <label className="label p-0 flex-1 min-w-0">
            <span
              className={`text-${size} font-medium text-secondary-700 line-clamp-2 leading-snug ${required && "after:content-['*'] after:ml-0.5 after:text-danger-500"}`}
            >
              {label}
            </span>
            {errorText && <span className="label-text-alt text-error">{errorText}</span>}
          </label>
        )}

        <div className="flex flex-wrap gap-2">
          {emails.map((email, index) => (
            <div key={index} className={`flex items-center mb-2 gap-1 px-3 py-1 rounded-full text-sm ${className}`}>
              <span>{email}</span>
              <Button
                variantType="secondary"
                variant="ghost"
                rounded
                size="2xs"
                iconComponent="icon-[ic--baseline-close]"
                onClick={() => removeEmail(index)}
              />
            </div>
          ))}
        </div>
      </div>

      <Input
        type="text"
        value={currentInput}
        onChange={handleInputChange}
        onBlur={addEmail}
        placeholder={placeholder}
        className="w-full outline-none text-sm text-gray-700"
        label=""
        errorText={errorText}
      />
    </div>
  );
};
